/***************************************************************
 * Name:      ActiveStereoMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Andy Barry (abarry@gmail.com)
 * Created:   2008-12-11
 * Copyright: Andy Barry ()
 * License:
 **************************************************************/

#include "ActiveStereoMain.h"
#include <wx/msgdlg.h>

//(*InternalHeaders(ActiveStereoFrame)
#include <wx/bitmap.h>
#include <wx/intl.h>
#include <wx/image.h>
#include <wx/string.h>
//*)

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

//(*IdInit(ActiveStereoFrame)
const long ActiveStereoFrame::ID_TEXTCTRL1 = wxNewId();
const long ActiveStereoFrame::ID_BUTTON2 = wxNewId();
const long ActiveStereoFrame::ID_SPINCTRL1 = wxNewId();
const long ActiveStereoFrame::ID_BUTTON1 = wxNewId();
const long ActiveStereoFrame::ID_BUTTON4 = wxNewId();
const long ActiveStereoFrame::ID_BUTTON3 = wxNewId();
const long ActiveStereoFrame::ID_PANEL1 = wxNewId();
const long ActiveStereoFrame::idMenuQuit = wxNewId();
const long ActiveStereoFrame::idMenuAbout = wxNewId();
const long ActiveStereoFrame::ID_STATUSBAR1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(ActiveStereoFrame,wxFrame)
    //(*EventTable(ActiveStereoFrame)
    //*)

    // OpenCV events
    EVT_COMMAND(wxID_ANY, IMAGE_UPDATE_EVENT, ActiveStereoFrame::UpdateImage)
	EVT_COMMAND(wxID_ANY, DISPLAY_TEXT_EVENT, ActiveStereoFrame::DisplayText)
	EVT_COMMAND(wxID_ANY, WAIT_FOR_LASER_EVENT, ActiveStereoFrame::WaitForLaser)
	EVT_COMMAND(wxID_ANY, CALIBRATE_EVENT, ActiveStereoFrame::CalibrateEvent)
	EVT_COMMAND(wxID_ANY, CALIBRATE_MOVE_OBJECT_EVENT, ActiveStereoFrame::MoveCalibrateObject)
END_EVENT_TABLE()

ActiveStereoFrame::ActiveStereoFrame(wxWindow* parent,wxWindowID id)
{
    //(*Initialize(ActiveStereoFrame)
    wxMenuItem* MenuItem2;
    wxMenuItem* MenuItem1;
    wxBoxSizer* BoxSizer2;
    wxMenu* Menu1;
    wxBoxSizer* BoxSizer1;
    wxMenuBar* MenuBar1;
    wxMenu* Menu2;

    Create(parent, id, _("Active Stereo"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, _T("id"));
    SetClientSize(wxSize(652,302));
    {
    wxIcon FrameIcon;
    FrameIcon.CopyFromBitmap(wxBitmap(wxImage(_T("/home/abarry/programs/ActiveStereo/laserIcon.png"))));
    SetIcon(FrameIcon);
    }
    Panel1 = new wxPanel(this, ID_PANEL1, wxPoint(120,232), wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_PANEL1"));
    BoxSizer1 = new wxBoxSizer(wxVERTICAL);
    txtLog = new wxTextCtrl(Panel1, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxSize(390,154), wxTE_MULTILINE|wxTE_READONLY|wxTE_WORDWRAP, wxDefaultValidator, _T("ID_TEXTCTRL1"));
    txtLog->SetMinSize(wxSize(390,154));
    BoxSizer1->Add(txtLog, 1, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    BoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
    butSetConfig = new wxButton(Panel1, ID_BUTTON2, _("Configure Laser"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON2"));
    BoxSizer2->Add(butSetConfig, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    spinLaser = new wxSpinCtrl(Panel1, ID_SPINCTRL1, _T("0"), wxDefaultPosition, wxSize(95,29), 0, 50, 150, 0, _T("ID_SPINCTRL1"));
    spinLaser->SetValue(_T("0"));
    spinLaser->Disable();
    BoxSizer2->Add(spinLaser, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    butGo = new wxButton(Panel1, ID_BUTTON1, _("Move Laser"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
    butGo->Disable();
    BoxSizer2->Add(butGo, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    butCalibrate = new wxButton(Panel1, ID_BUTTON4, _("Calibrate"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON4"));
    butCalibrate->Disable();
    BoxSizer2->Add(butCalibrate, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    butCapture = new wxButton(Panel1, ID_BUTTON3, _("Start Scan"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON3"));
    butCapture->Disable();
    BoxSizer2->Add(butCapture, 1, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    BoxSizer1->Add(BoxSizer2, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    Panel1->SetSizer(BoxSizer1);
    BoxSizer1->Fit(Panel1);
    BoxSizer1->SetSizeHints(Panel1);
    MenuBar1 = new wxMenuBar();
    Menu1 = new wxMenu();
    MenuItem1 = new wxMenuItem(Menu1, idMenuQuit, _("Quit\tAlt-F4"), _("Quit the application"), wxITEM_NORMAL);
    Menu1->Append(MenuItem1);
    MenuBar1->Append(Menu1, _("&File"));
    Menu2 = new wxMenu();
    MenuItem2 = new wxMenuItem(Menu2, idMenuAbout, _("About\tF1"), _("Show info about this application"), wxITEM_NORMAL);
    Menu2->Append(MenuItem2);
    MenuBar1->Append(Menu2, _("Help"));
    SetMenuBar(MenuBar1);
    StatusBar1 = new wxStatusBar(this, ID_STATUSBAR1, 0, _T("ID_STATUSBAR1"));
    int __wxStatusBarWidths_1[1] = { -1 };
    int __wxStatusBarStyles_1[1] = { wxSB_NORMAL };
    StatusBar1->SetFieldsCount(1,__wxStatusBarWidths_1);
    StatusBar1->SetStatusStyles(1,__wxStatusBarStyles_1);
    SetStatusBar(StatusBar1);

    Connect(ID_BUTTON2,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ActiveStereoFrame::OnButSetConfigClick);
    Connect(ID_BUTTON1,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ActiveStereoFrame::OnButGoClick);
    Connect(ID_BUTTON4,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ActiveStereoFrame::OnButCalibrateClick);
    Connect(ID_BUTTON3,wxEVT_COMMAND_BUTTON_CLICKED,(wxObjectEventFunction)&ActiveStereoFrame::OnButCaptureClick);
    Connect(idMenuQuit,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&ActiveStereoFrame::OnQuit);
    Connect(idMenuAbout,wxEVT_COMMAND_MENU_SELECTED,(wxObjectEventFunction)&ActiveStereoFrame::OnAbout);
    //*)

    // OpenCV GUI creation
    cvNamedWindow("Output1", CV_WINDOW_AUTOSIZE);
	cvNamedWindow("Output2", CV_WINDOW_AUTOSIZE);

	statMax = 0;
	statMin = -1;
	averageSum = 0;
	averageNum = 0;
}

ActiveStereoFrame::~ActiveStereoFrame()
{
    //(*Destroy(ActiveStereoFrame)
    //*)
}

void ActiveStereoFrame::OnQuit(wxCommandEvent& event)
{
    Close();
}

void ActiveStereoFrame::OnAbout(wxCommandEvent& event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Welcome to..."));
}

void ActiveStereoFrame::OnButGoClick(wxCommandEvent& event)
{
    txtLog->AppendText(wxT("\nMoving laser..."));
    int num = spinLaser->GetValue();
    int res = SetHorz(num);
    switch (res)
    {
		case 0:
			txtLog->AppendText(wxT(" success."));
			break;
        case -1:
            txtLog->AppendText(wxT(" no matching device found."));
            break;
        case -2:
            txtLog->AppendText(wxT(" unable to send request."));
            break;
		case -3:
			txtLog->AppendText(wxT(" invalid offset."));
			break;
    }
}

void ActiveStereoFrame::OnButSetConfigClick(wxCommandEvent& event)
{
    txtLog->AppendText(wxT("\nConfiguring... "));
    int res = SetConfig();
    switch (res)
    {
        case 0:
			txtLog->AppendText(wxT(" success."));
			spinLaser->Enable(true);
			butGo->Enable(true);
			butCapture->Enable(true);
			butCalibrate->Enable(true);
			break;
        case -1:
            txtLog->AppendText(wxT(" no matching device found."));
            break;
        case -2:
            txtLog->AppendText(wxT(" unable to send SET_CONFIGURATION standard request."));
            break;
    }
}

void ActiveStereoFrame::OnButCaptureClick(wxCommandEvent& event)
{
	if (!cam)
	{
		cam = new Cameras(txtLog, this);  // create the camera object if it doesn't already exist
	}
	cam->Play();
}

void ActiveStereoFrame::UpdateImage(wxCommandEvent &event)
{
	// update image display
	IplImage *img = (IplImage*)event.GetClientData();
	cvShowImage(event.GetString().char_str(), img);
}

void ActiveStereoFrame::DisplayText(wxCommandEvent &event)
{
	//txtLog->AppendText(event.GetString());
	double* num2 = (double*)event.GetClientData();
	double num = *num2;
	delete num2;
	if (num == -2)
	{
		wxString summary = wxT("\n-----------\nSummary --\nMax: ");
		summary << statMax;
		summary += wxT("\nMin: ");
		summary << statMin;
		summary += wxT("\nAverage: ");
		summary << (averageSum / averageNum);
		txtLog->AppendText(summary);
		statMax = 0;
		statMin = -1;
		averageSum = 0;
		averageNum = 0;
	}
	if (num > 0)
	{
		if (num > statMax)
		{
			statMax = num;
		}
		if (num < statMin || statMin == -1)
		{
			statMin = num;
		}
		averageSum += num;
		averageNum ++;
	}

}

void ActiveStereoFrame::WaitForLaser(wxCommandEvent &event)
{
	((wxMutex*)event.GetClientData())->Lock();
	wxMessageBox(event.GetString());

	// allow the thread to continue
	if (((wxMutex*)event.GetClientData())->Unlock() == wxMUTEX_UNLOCKED)
	{
		txtLog->AppendText(wxT("\nDon't own mutex."));
	}
}

void ActiveStereoFrame::OnButCalibrateClick(wxCommandEvent& event)
{
	// get the initial distance

	calibrateDistance = GetCalibrateDistance(100);

	calString = wxT("");
	calString << calibrateDistance;
	calString += wxT(">");

	calString2 = wxT("");

	if (!cam)
	{
		cam = new Cameras(txtLog, this);  // create the camera object if it doesn't already exist
	}
	cam->SetCalibrate(true);
	cam->Play();
}

void ActiveStereoFrame::CalibrateEvent(wxCommandEvent &event)
{
	// catch a calibrate data point.

	// write the data point to a file, associated with the distance we are currently at
	int xPos = event.GetInt();
	int laserPos = event.GetExtraLong();

	calString << laserPos;
	calString += wxT(",");
	calString << xPos;
	calString += wxT("|");

	// matlab formatted stuff

	calString2 += wxT("\narray(");
	calString2 << calibrateDistance;
	calString2 += wxT(",");
	calString2 << laserPos;
	calString2 += wxT(") = ");
	calString2 << xPos;
	calString2 += wxT(";");


	/*
	* File Syntax:
	* ------------
	* Distance (cm)>Angle,xPos|Angle,xPos|Angle,xPos
	* Distance (cm)>Angle,xPos|Angle,xPos|Angle,xPos
	* Distance (cm)>Angle,xPos|Angle,xPos|Angle,xPos
	* ------------
	*/
}

void ActiveStereoFrame::MoveCalibrateObject(wxCommandEvent &event)
{
	((wxMutex*)event.GetClientData())->Lock();

	// write last line to file

	wxFFile file(wxT("calibration.dat"), wxT("a"));
	file.Write(calString);

	// wait for the new distance
	calibrateDistance = GetCalibrateDistance(calibrateDistance);
	calString = wxT("\n");
	calString << calibrateDistance;
	calString += wxT(">");
	file.Close();

    wxFFile file2(wxT("calibrationMat.dat"), wxT("a"));
	file2.Write(calString2);
	file2.Close();

	// allow the thread to continue
	if (((wxMutex*)event.GetClientData())->Unlock() == wxMUTEX_UNLOCKED)
	{
		txtLog->AppendText(wxT("\nDon't own mutex."));
	}

}

int ActiveStereoFrame::GetCalibrateDistance(int lastDistance)
{
	while (true)
	{
		wxNumberEntryDialog dialog(this, wxT("Please enter the distance to the calibration object in centimetres."),
			wxT("Distance to object (cm):"), wxT("Active Stereo Calibration"), lastDistance, 1, 10000);
		if (dialog.ShowModal() == wxID_OK)
		{
			return dialog.GetValue();
		}
	}
}

