#include "Cameras.h"


Cameras::Cameras(wxTextCtrl *pMemo, wxFrame *windowIn)
{
	window = windowIn;
	m_MyCapture = NULL;
	m_LastFrame = NULL;
	m_pMemo = pMemo;
	m_CamPaused = false;
	m_CamPlaying = false;
	m_CamStopped = false;

	// filename format for the first frame from camera 0 is MyFrame0.bmp
	// the 12th frame is MyFrame12.bmp
	m_filename =  wxT("MyFrame");   // This is the base name, integers will be appended for each frame captured
	m_fileformat = wxT(".bmp");	// can choose bmp, jpg, etc for saving images
	m_LastCapturedFrameFilename = wxT("");
	m_FilenameIndex = wxT("");
	m_NumberOfCapturedFrames = 0;

	cvNamedWindow("My Camera", CV_WINDOW_AUTOSIZE);

	InitializeCamera();


}

Cameras::~Cameras()
{
	Stop();
	if (m_MyCapture) cvReleaseCapture(&m_MyCapture);
	if (m_LastFrame) cvReleaseImage(&m_LastFrame);
	cvDestroyWindow("My Camera");
	calibrate = false;
}



void Cameras::InitializeCamera()
{
	m_MyCapture = cvCreateCameraCapture(CV_CAP_ANY);
	if (!m_MyCapture)
	{
		m_pMemo->AppendText(wxT("\n\nFAILED to get capture from camera!!"));
		return;
	}
	cvQueryFrame(m_MyCapture); // this call is necessary to get correct capture properties
	m_FrameHeight   = (int) cvGetCaptureProperty(m_MyCapture, CV_CAP_PROP_FRAME_HEIGHT);
	m_FrameWidth    = (int) cvGetCaptureProperty(m_MyCapture, CV_CAP_PROP_FRAME_WIDTH);
	m_FrameRate     = (int) cvGetCaptureProperty(m_MyCapture, CV_CAP_PROP_FPS);
	wxString TempString = wxT("\nFrame Height, Width, Rate = ");
	TempString << m_FrameHeight << wxT(", ") <<	m_FrameWidth << wxT(", ") << m_FrameRate;
	m_pMemo->AppendText(TempString);
	m_LastFrame = cvQueryFrame(m_MyCapture);
	cvShowImage("MyCamera", m_LastFrame);
}

bool Cameras::CaptureExists()
{
	if (!m_MyCapture)
	{
		m_pMemo->AppendText(wxT("\nCamera not initialized."));
		return false;
	}
	return true;
}


void Cameras::Play()
{
	if ( (!CaptureExists()) || m_CamPlaying) return;

	//int WaitTime = (int) ((1000. / (float) (m_FrameRate)) + 0.5);
	m_CamStopped = false;
	m_CamPaused = false;
	m_CamPlaying = true;

	ScanThread *myScanThread = new ScanThread(window, m_MyCapture);
	if ( myScanThread->Create() != wxTHREAD_NO_ERROR )
	{
		m_pMemo->AppendText(wxT("\nFailed to create capture thread."));
	} else {
		myScanThread->SetCalibrate(calibrate);
		myScanThread->Run();
	}
	/*
	while(1)
	{
		m_LastFrame = cvQueryFrame(m_MyCapture);
		if (!m_LastFrame) break;
		cvShowImage("My Camera", m_LastFrame);
		cvWaitKey(33);   // (WaitTime);
		if (m_CamStopped || m_CamPaused) break;
	}
	*/
}

void Cameras::Pause()
{
	if (!CaptureExists()) return;
	m_CamPaused = true;
	m_CamStopped = false;
	m_CamPlaying = false;
}

void Cameras::Stop()
{
	if (!CaptureExists()) return;
	m_CamStopped = true;
	m_CamPaused = false;
	m_CamPlaying = false;
}


IplImage *Cameras::FrameGrab()
{
	if (!CaptureExists()) return NULL;
	Stop();
	m_LastFrame = cvQueryFrame(m_MyCapture);
	cvShowImage("My Camera", m_LastFrame);
	return m_LastFrame;
}

void Cameras::SaveSingleFrame()
{
	if (!CaptureExists()) return;
	Stop();
	m_NumberOfCapturedFrames++;
	cvSaveImage(GetLastCapturedFrameFilename().char_str(), m_LastFrame);
	//m_pMemo->AppendText(wxChar((char*)GetLastCapturedFrameFilename().c_str()));
	//cvSaveImage("testImage.bmp", m_LastFrame);
}


wxString Cameras::GetLastCapturedFrameFilename()
{
	m_LastCapturedFrameFilename = m_filename;	// basic filename
	m_LastCapturedFrameFilename << m_NumberOfCapturedFrames;
	m_LastCapturedFrameFilename.Append(m_fileformat);
		m_pMemo->AppendText(m_LastCapturedFrameFilename);

	return m_LastCapturedFrameFilename;
}

