#ifndef SCAN_THREAD_H
#define SCAN_THREAD_H

#include <wx/thread.h>
#include <wx/string.h>
#include <wx/textctrl.h>
#include <wx/dialog.h>
#include <wx/wx.h>
#include "cv.h"
#include "highgui.h"
#include "IndividualPixels.h"

#include <wx/dynarray.h>
#include <wx/ffile.h>

WX_DECLARE_OBJARRAY(wxPoint*, ArrayOfPoints);

WX_DECLARE_HASH_MAP( int, ArrayOfPoints*, wxIntegerHash, wxIntegerEqual, CalMap );


extern "C"
{
    #include "SetHorz.h"
}

#define LASER_MAX_LEFT 220
#define LASER_MAX_RIGHT 20

// in the header of the source file
BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE(IMAGE_UPDATE_EVENT, -1)
	DECLARE_EVENT_TYPE(DISPLAY_TEXT_EVENT, -2)
	DECLARE_EVENT_TYPE(WAIT_FOR_LASER_EVENT, -3)
	DECLARE_EVENT_TYPE(CALIBRATE_EVENT, -4)
	DECLARE_EVENT_TYPE(CALIBRATE_MOVE_OBJECT_EVENT, -5)

END_DECLARE_EVENT_TYPES()





class ScanThread : public wxThread
{
	public:
		ScanThread(wxFrame *windowIn, CvCapture *capture);
		virtual void* Entry();

		void SetCalibrate(bool x) { calibrate = x; }

	private:
		IplImage* FindLaser(IplImage *noLaser, IplImage *withLaser, IplImage *scanOutput, int laserPos);
		void DisplayImage(IplImage *frame, wxString str);
		void WaitForLaser(bool on);
		void WaitForCalibrate();
		void DisplayText(wxString text, double num = -1);
		double PixelToDistance(int xPos, int laserPos);
		int DistanceToColor(double dist);
		void SendCalibrationEvent(int xPos, int laserPos);
		bool LoadCalibrationData();
		void OnExit();

		CalMap *calData;

		wxFrame *window;
		CvCapture *cvCapture;
		bool calibrate;

};


#endif
