#include <stdio.h>
#include <usb.h>

#include "SetConfig.h"

#define FALSE       0
#define TRUE        1

// return codes
// 0: success
// -1: No matching device found
// -2: Unable to send SET_CONFIGURATION request

int SetConfig() {
    struct usb_bus *bus;
    struct usb_device *dev;
    usb_dev_handle *udev;
    int dev_found, ret;
    unsigned char buffer[8];

    usb_init();

    usb_find_busses();
    usb_find_devices();

    udev = NULL;
    dev_found = FALSE;
    for (bus = usb_get_busses(); bus && !dev_found; bus = bus->next) {
        for (dev = bus->devices; dev && !dev_found; dev = dev->next) {
            if ((dev->descriptor.idVendor == 0x6666) && (dev->descriptor.idProduct == 0x0003)) {
                dev_found = TRUE;
                udev = usb_open(dev);
            }
        }
    }
    if (!dev_found) {
        printf("No matching device found...\n");
        return -1;
    }
    if (udev) {
        ret = usb_control_msg(udev, USB_TYPE_STANDARD | USB_RECIP_DEVICE, USB_REQ_SET_CONFIGURATION, 1, 0, buffer, 0, 100);
        if (ret < 0) {
            printf("Unable to send SET_CONFIGURATION standard request, ret = %d...\n", ret);
            return -2;
        usb_close(udev);
        }
    }
    return 0;
}
