#include <usb.h>
#include <stdio.h>

#include "SetHorz.h"

#define FALSE       0
#define TRUE        1

#define SET_DUTY    0x04

/*
* Return codes:
*	 0 = success
*	-1 = no matching device
*	-2 = unable to send vendor request
*	-3 = illegal offset
*/

int SetHorz(int horz)
{
    struct usb_bus *bus;
    struct usb_device *dev;
    usb_dev_handle *udev;
    int dev_found, ret;
    unsigned char buffer[8];

    if ((horz < 20) || (horz > 220)) {
        printf("Illegal offset (%d) specified.\n", horz);
		return -3;
    }

    usb_init();

    usb_find_busses();
    usb_find_devices();

    udev = NULL;
    dev_found = FALSE;
    for (bus = usb_get_busses(); bus && !dev_found; bus = bus->next) {
        for (dev = bus->devices; dev && !dev_found; dev = dev->next) {
            if ((dev->descriptor.idVendor == 0x6666) && (dev->descriptor.idProduct == 0x0003)) {
                dev_found = TRUE;
                udev = usb_open(dev);
            }
        }
    }
    if (!dev_found) {
        printf("No matching device found...\n");
		return -1;
    }

    if (udev) {
        ret = usb_control_msg(udev, 0x40, SET_DUTY, horz, 0, buffer, 0, 100);
        if (ret < 0) {
            printf("Unable to send SET_DUTY vendor request, ret = %d...\n", ret);
            return -2;
        }
        //usb_close(udev);
    }
    return 0;
}
